"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFlagValue = exports.getConfigValue = void 0;
const State_1 = require("./State");
const index_1 = require("./index");
const ENVIRONMENT_VARIABLE_PREFIX = 'GCB_';
function getConfigValue(name, defaultValue) {
    // Try to get config value from environment variable.
    const envVariable = ENVIRONMENT_VARIABLE_PREFIX + name.toUpperCase();
    const envValue = process.env[envVariable];
    const argsValue = State_1.args[name.toLowerCase()];
    // getConfig can be undefined during the first few calls to this function because the build config is initialized
    // before the getConfig function is defined. In those cases, a defaultValue is provided.
    const configValue = ((index_1.getConfig ? (0, index_1.getConfig)() : {}) || {})[name];
    return _firstDefinedValue(argsValue, envValue, defaultValue, configValue);
}
exports.getConfigValue = getConfigValue;
function getFlagValue(name, defaultValue) {
    const configValue = getConfigValue(name, defaultValue);
    return configValue === 'true' || configValue === true;
}
exports.getFlagValue = getFlagValue;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function _firstDefinedValue(...values) {
    for (const value of values) {
        if (value !== undefined) {
            return value;
        }
    }
    return undefined;
}
//# sourceMappingURL=config.js.map